#!/bin/bash
#-------------------------------------------------------------------------------
# Utility script for fetching geolocation databases.
#
# Copyright (C) since 2011 CESNET, z.s.p.o
# Use of this source is governed by the MIT license, see LICENSE file.
#-------------------------------------------------------------------------------


#
# Helper functions for displaying information of various severity to the user.
#
function error {
    /bin/echo -e "${RED}[ERROR] $1${NC}" 1>&2
    exit 1
}

function info {
    if [ $FLAG_QUIET != 'YES' ]; then
        /bin/echo "$1"
    fi
}

function success {
    if [ $FLAG_QUIET != 'YES' ]; then
        /bin/echo -e "${GREEN}[SUCCESS] $1${NC}"
    fi
    exit 0
}

function usage {
    /bin/echo ""
    /bin/echo "Utility script for fetching geolocation databases."
    /bin/echo ""
    /bin/echo "Available options:"
    /bin/echo "    -h | --help    display this help message and exit"
    /bin/echo "    -q | --quiet   run in quiet mode, display only errors"
    /bin/echo ""
    /bin/echo "    -t=dir"
    /bin/echo "    --target=dir   name of the target directory (defaults to '/var/opt/opensourcedbs')"
    /bin/echo ""
}

#-------------------------------------------------------------------------------


#
# Color code definitions for colored terminal output.
# Source: https://stackoverflow.com/questions/5947742/how-to-change-the-output-color-of-echo-in-linux
#
RED='\033[0;31m'
GREEN='\033[0;32m'
ORANGE='\033[0;33m'
NC='\033[0m'

FLAG_DEVEL=NO
FLAG_QUIET=NO
DBDIR="/var/opt/opensourcedbs"
STAMPF=".download.ts"


#-------------------------------------------------------------------------------


#
# Command line argument parsing.
# Source: https://stackoverflow.com/questions/192249/how-do-i-parse-command-line-arguments-in-bash
#
for i in "$@"
do
case $i in
    -h|--help)
    usage
    exit 0
    ;;
    -q|--quiet)
    FLAG_QUIET=YES
    shift # past argument=value
    ;;
    -t=*|--target=*)
    DBDIR="${i#*=}"
    shift # past argument=value
    ;;
    *)
    usage
    error "Invalid usage, unknown command line option '$i'"
    ;;
esac
done


#-------------------------------------------------------------------------------


info "Fetching database files into folder: '$DBDIR'"

mkdir -p $DBDIR

#
# Limit downloads to once in 12 hours.
#
if [ -f "$DBDIR/GeoLite2-ASN.mmdb" ] && [ -f "$DBDIR/$STAMPF" ]
then
    if [ "$(( $(date +'%s') - $(date -r $DBDIR/$STAMPF +'%s') ))" -lt "43200" ]
    then
        success "All database files in '$DBDIR' are recent enough at `date` (last update at `date -r $DBDIR/$STAMPF`)"
    else
        info "Database file were last updated at `date -r $DBDIR/$STAMPF`"
    fi
fi

#
# Fetch latest versions of GeoLite2 database files.
#
/usr/bin/wget -q http://geolite.maxmind.com/download/geoip/database/GeoLite2-ASN.tar.gz -O "$DBDIR/GeoLite2-ASN.tar.gz"
/usr/bin/wget -q http://geolite.maxmind.com/download/geoip/database/GeoLite2-City.tar.gz -O "$DBDIR/GeoLite2-City.tar.gz"
/usr/bin/wget -q http://geolite.maxmind.com/download/geoip/database/GeoLite2-Country.tar.gz -O "$DBDIR/GeoLite2-Country.tar.gz"

#
# Fetch latest version of ISO 3166-2 country code database.
#
/usr/bin/wget -q https://raw.githubusercontent.com/datasets/country-codes/master/data/country-codes.csv -O "$DBDIR/country-codes.csv"

#
# Remove obsolete versions of database files.
#
/bin/rm -f "$DBDIR/GeoLite2-ASN.mmdb"
/bin/rm -f "$DBDIR/GeoLite2-City.mmdb"
/bin/rm -f "$DBDIR/GeoLite2-Country.mmdb"

cd $DBDIR

#
# Extract, install and cleanup.
#
if [ $FLAG_QUIET == 'YES' ]; then
    /bin/tar -xzf GeoLite2-ASN*.tar.gz
    /bin/tar -xzf GeoLite2-City*.tar.gz
    /bin/tar -xzf GeoLite2-Country*.tar.gz
else
    /bin/tar -xzvf GeoLite2-ASN*.tar.gz
    /bin/tar -xzvf GeoLite2-City*.tar.gz
    /bin/tar -xzvf GeoLite2-Country*.tar.gz
fi

/bin/mv GeoLite2-ASN_*/GeoLite2-ASN.mmdb .
/bin/mv GeoLite2-City_*/GeoLite2-City.mmdb .
/bin/mv GeoLite2-Country_*/GeoLite2-Country.mmdb .

/bin/rm -rf GeoLite2-ASN_*
/bin/rm -rf GeoLite2-City_*
/bin/rm -rf GeoLite2-Country_*

/bin/rm -f GeoLite2-ASN.tar.gz
/bin/rm -f GeoLite2-City.tar.gz
/bin/rm -f GeoLite2-Country.tar.gz

#
# Check, that database files really exist on the filesystem.
#
GEOIPFETCHDBRESULT='success'
if [ -f "$DBDIR/GeoLite2-ASN.mmdb" ]
then
    info "GeoLite2 ASN database file downloaded and installed"
else
    info "Installation of GeoLite2 ASN database failed"
    GEOIPFETCHDBRESULT="failure"
fi
if [ -f "$DBDIR/GeoLite2-City.mmdb" ]
then
    info "GeoLite2 City database file downloaded and installed"
else
    info "Installation of GeoLite2 City database failed"
    GEOIPFETCHDBRESULT="failure"
fi
if [ -f "$DBDIR/GeoLite2-Country.mmdb" ]
then
    info "GeoLite2 Country database file downloaded and installed"
else
    info "Installation of GeoLite2 Country database failed"
    GEOIPFETCHDBRESULT="failure"
fi

#
# Report overall status.
#
if [ "x${GEOIPFETCHDBRESULT}x" = "xsuccessx" ]
then
    /usr/bin/touch "$DBDIR/$STAMPF"
    success "All database files downloaded and installed into '$DBDIR' at `date`"
else
    error "Installation of database files into '$DBDIR' failed"
fi

